@@ -2,11 +2,13 @@ pub mod actions;
 pub mod activity;
 pub mod apps;
 pub mod checks;
+pub mod code_scannings;
 pub mod commits;
 pub mod current;
 pub mod events;
 pub mod gists;
 pub mod gitignore;
+pub mod hooks;
 pub mod issues;
 pub mod licenses;
 pub mod markdown;
