@@ -1,17 +1,15 @@
 //! The pull request API.
 
-mod comment;
-mod create;
-mod list;
-mod merge;
-mod update;
-
 use http::request::Builder;
 use http::{Method, Uri};
-
+use serde_json::json;
 use snafu::ResultExt;
 
 use crate::error::HttpSnafu;
+use crate::models::pulls::ReviewComment;
+use crate::models::CommentId;
+use crate::pulls::specific_pr::pr_reviews::specific_review::SpecificReviewBuilder;
+use crate::pulls::specific_pr::{SpecificPullRequestBuilder, SpecificPullRequestCommitBuilder};
 use crate::{Octocrab, Page};
 
 pub use self::{
@@ -19,6 +17,13 @@ pub use self::{
     update::UpdatePullRequestBuilder,
 };
 
+mod comment;
+mod create;
+mod list;
+mod merge;
+mod specific_pr;
+mod update;
+
 /// A client to GitHub's pull request API.
 ///
 /// Created with [`Octocrab::pulls`].
@@ -328,13 +333,16 @@ impl<'octo> PullRequestHandler<'octo> {
         self.crab.delete(route, Some(&map)).await
     }
 
-    /// List all `FileDiff`s associated with the pull request.
+    /// List all `DiffEntry`s associated with the pull request.
     /// ```no_run
     /// # async fn run() -> octocrab::Result<()> {
     /// let files = octocrab::instance().pulls("owner", "repo").list_files(101).await?;
     /// # Ok(())
     /// # }
-    pub async fn list_files(&self, pr: u64) -> crate::Result<Page<crate::models::pulls::FileDiff>> {
+    pub async fn list_files(
+        &self,
+        pr: u64,
+    ) -> crate::Result<Page<crate::models::repos::DiffEntry>> {
         let route = format!(
             "/repos/{owner}/{repo}/pulls/{pr}/files",
             owner = self.owner,
@@ -369,6 +377,107 @@ impl<'octo> PullRequestHandler<'octo> {
         comment::ListCommentsBuilder::new(self, pr)
     }
 
+    ///creates a new `CommentBuilder` for GET/PATCH/DELETE requests
+    /// to the `/repos/{owner}/{repo}/pulls/{pr}/comments/{comment_id}` endpoint
+    /// ```no_run
+    ///  use octocrab::models::CommentId;
+    /// use octocrab::models::pulls::Comment;
+    ///  async fn run() -> octocrab::Result<Comment> {
+    ///     let octocrab = octocrab::Octocrab::default();
+    ///     let _ = octocrab.pulls("owner", "repo").comment(CommentId(21)).delete();
+    ///     let _ = octocrab.pulls("owner", "repo").comment(CommentId(42)).update("new comment");
+    ///     let comment = octocrab.pulls("owner", "repo").comment(CommentId(42)).get().await;
+    ///
+    ///     comment
+    ///  }
+    /// ```
+    pub fn comment(&self, comment_id: CommentId) -> comment::CommentBuilder {
+        comment::CommentBuilder::new(self, comment_id)
+    }
+
+    /// creates a builder for the `/repos/{owner}/{repo}/pulls/{pull_number}/......` endpoint
+    /// working with particular pull request, e.g.
+    /// * /repos/{owner}/{repo}/pulls/{pull_number}/reviews/{review_id}/events
+    /// * /repos/{owner}/{repo}/pulls/{pull_number}/reviews/{review_id}
+    /// * /repos/{owner}/{repo}/pulls/{pull_number}/commits
+    /// * /repos/{owner}/{repo}/pulls/{pull_number}/reviews/{review_id}/comments
+    /// * /repos/{owner}/{repo}/pulls/{pull_number}/reviews/{review_id}/dismissals
+    /// * /repos/{owner}/{repo}/pulls/{pull_number}/comments/{comment_id}/replies
+    ///
+    #[deprecated(
+        since = "0.34.4",
+        note = "specific PR builder transitioned to pr_review_actions, pr_commits, reply_to_comment"
+    )]
+    //FIXME: remove?
+    pub fn pull_number(&self, pull_nr: u64) -> SpecificPullRequestBuilder {
+        SpecificPullRequestBuilder::new(self, pull_nr)
+    }
+
+    // /repos/{owner}/{repo}/pulls/{pull_number}/reviews/{review_id}/events
+    // /repos/{owner}/{repo}/pulls/{pull_number}/reviews/{review_id}
+    // repos/{owner}/{repo}/pulls/{pull_number}/reviews/{review_id}/comments
+    // repos/{owner}/{repo}/pulls/{pull_number}/reviews/{review_id}/dismissals
+    pub fn pr_review_actions(
+        &self,
+        pull_nr: u64,
+        review_id: u64,
+    ) -> SpecificReviewBuilder<'octo, '_> {
+        SpecificReviewBuilder::new(self, pull_nr, review_id)
+    }
+
+    /// Lists a maximum of 250 commits for a pull request.
+    ///
+    /// To receive a complete commit list for pull requests with more than 250 commits,
+    /// use the [List commits](https://docs.github.com/rest/commits/commits#list-commits) endpoint.
+    ///
+    /// ```no_run
+    /// # async fn run() -> octocrab::Result<()> {
+    /// let commits = octocrab::instance()
+    ///     .pulls("owner", "repo")
+    ///     .pr_commits(21u64)
+    ///     .per_page(100)
+    ///     .page(2u32)
+    ///     .send()
+    ///     .await?;
+    /// # Ok(())
+    /// # }
+    /// ```
+    // /repos/{owner}/{repo}/pulls/{pull_number}/commits
+    pub fn pr_commits(&self, pr_number: u64) -> SpecificPullRequestCommitBuilder<'_, '_> {
+        SpecificPullRequestCommitBuilder::new(self, pr_number)
+    }
+
+    // /repos/{owner}/{repo}/pulls/{pull_number}/comments/{comment_id}/replies
+    /// Creates a reply to a specific comment of a pull request specified in the first argument
+    /// ```no_run
+    /// # use octocrab::models::CommentId;
+    ///  async fn run() -> octocrab::Result<()> {
+    /// # let octocrab = octocrab::Octocrab::default();
+    /// use octocrab::params;
+    ///
+    /// let page = octocrab.pulls("owner", "repo").reply_to_comment(142, CommentId(24), "This is my reply")
+    ///     .await?;
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub async fn reply_to_comment(
+        &self,
+        pull_nr: u64,
+        comment_id: CommentId,
+        comment: impl Into<String>,
+    ) -> crate::Result<ReviewComment> {
+        let route = format!(
+            "/repos/{owner}/{repo}/pulls/{pull_number}/comments/{comment_id}/replies",
+            owner = self.owner,
+            repo = self.repo,
+            pull_number = pull_nr,
+            comment_id = comment_id
+        );
+        self.crab
+            .post(route, Some(&json!({ "body": comment.into() })))
+            .await
+    }
+
     /// Creates a new `MergePullRequestsBuilder` that can be configured used to
     /// merge a pull request.
     /// ```no_run
