@@ -1,20 +1,26 @@
 //! The repositories API.
 
+use bytes::Bytes;
 use http::header::ACCEPT;
 use http::request::Builder;
 use http::Uri;
+use http_body_util::combinators::BoxBody;
 use snafu::ResultExt;
 
 mod branches;
 mod collaborators;
 mod commits;
+mod contributors;
+mod dependabot;
 pub mod events;
 mod file;
 pub mod forks;
 mod generate;
 mod merges;
 mod pulls;
+pub mod release_assets;
 pub mod releases;
+mod secret_scanning_alerts;
 mod secrets;
 mod stargazers;
 mod status;
@@ -22,34 +28,54 @@ mod tags;
 mod teams;
 
 use crate::error::HttpSnafu;
+use crate::models::commits::GitCommitObject;
+use crate::models::{repos, RepositoryId};
 use crate::repos::file::GetReadmeBuilder;
 use crate::{models, params, Octocrab, Result};
 pub use branches::ListBranchesBuilder;
 pub use collaborators::ListCollaboratorsBuilder;
 pub use commits::ListCommitsBuilder;
+pub use contributors::ListContributorsBuilder;
+pub use dependabot::RepoDependabotAlertsHandler;
 pub use file::{DeleteFileBuilder, GetContentBuilder, UpdateFileBuilder};
 pub use generate::GenerateRepositoryBuilder;
 pub use merges::MergeBranchBuilder;
 pub use pulls::ListPullsBuilder;
+pub use release_assets::ReleaseAssetsHandler;
 pub use releases::ReleasesHandler;
+pub use secret_scanning_alerts::RepoSecretScanningAlertsHandler;
 pub use secrets::RepoSecretsHandler;
 pub use stargazers::ListStarGazersBuilder;
 pub use status::{CreateStatusBuilder, ListStatusesBuilder};
 pub use tags::ListTagsBuilder;
 pub use teams::ListTeamsBuilder;
 
+#[derive(Clone)]
+pub(crate) enum RepoRef {
+    ByOwnerAndName(String, String),
+    ById(RepositoryId),
+}
+
+impl std::fmt::Display for RepoRef {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match self {
+            RepoRef::ByOwnerAndName(owner, name) => write!(f, "repos/{}/{}", owner, name),
+            RepoRef::ById(id) => write!(f, "repositories/{}", id),
+        }
+    }
+}
+
 /// Handler for GitHub's repository API.
 ///
 /// Created with [`Octocrab::repos`].
 pub struct RepoHandler<'octo> {
     crab: &'octo Octocrab,
-    owner: String,
-    repo: String,
+    repo: RepoRef,
 }
 
 impl<'octo> RepoHandler<'octo> {
-    pub(crate) fn new(crab: &'octo Octocrab, owner: String, repo: String) -> Self {
-        Self { crab, owner, repo }
+    pub(crate) fn new(crab: &'octo Octocrab, repo: RepoRef) -> Self {
+        Self { crab, repo }
     }
 
     /// Get's a repository's license.
@@ -60,11 +86,7 @@ impl<'octo> RepoHandler<'octo> {
     /// # }
     /// ```
     pub async fn license(&self) -> Result<models::repos::Content> {
-        let route = format!(
-            "/repos/{owner}/{repo}/license",
-            owner = self.owner,
-            repo = self.repo,
-        );
+        let route = format!("/{}/license", self.repo);
 
         self.crab.get(route, None::<&()>).await
     }
@@ -77,11 +99,7 @@ impl<'octo> RepoHandler<'octo> {
     /// # }
     /// ```
     pub async fn public_key(&self) -> Result<models::PublicKey> {
-        let route = format!(
-            "/repos/{owner}/{repo}/actions/secrets/public-key",
-            owner = self.owner,
-            repo = self.repo,
-        );
+        let route = format!("/{}/actions/secrets/public-key", self.repo);
 
         self.crab.get(route, None::<&()>).await
     }
@@ -97,11 +115,7 @@ impl<'octo> RepoHandler<'octo> {
     /// # }
     /// ```
     pub async fn get(&self) -> Result<models::Repository> {
-        let route = format!(
-            "/repos/{owner}/{repo}",
-            owner = self.owner,
-            repo = self.repo,
-        );
+        let route = format!("/{}", self.repo);
         self.crab.get(route, None::<&()>).await
     }
 
@@ -116,11 +130,7 @@ impl<'octo> RepoHandler<'octo> {
     /// # }
     /// ```
     pub async fn get_community_profile_metrics(&self) -> Result<models::RepositoryMetrics> {
-        let route = format!(
-            "/repos/{owner}/{repo}/community/profile",
-            owner = self.owner,
-            repo = self.repo,
-        );
+        let route = format!("/{}/community/profile", self.repo);
         self.crab.get(route, None::<&()>).await
     }
 
@@ -141,8 +151,7 @@ impl<'octo> RepoHandler<'octo> {
         reference: &params::repos::Reference,
     ) -> Result<models::repos::Ref> {
         let route = format!(
-            "/repos/{owner}/{repo}/git/ref/{reference}",
-            owner = self.owner,
+            "/{repo}/git/ref/{reference}",
             repo = self.repo,
             reference = reference.ref_url(),
         );
@@ -163,8 +172,7 @@ impl<'octo> RepoHandler<'octo> {
     /// ```
     pub async fn get_tag(&self, tag_sha: impl Into<String>) -> Result<models::repos::GitTag> {
         let route = format!(
-            "/repos/{owner}/{repo}/git/tags/{tag_sha}",
-            owner = self.owner,
+            "/{repo}/git/tags/{tag_sha}",
             repo = self.repo,
             tag_sha = tag_sha.into(),
         );
@@ -190,11 +198,7 @@ impl<'octo> RepoHandler<'octo> {
         reference: &params::repos::Reference,
         sha: impl Into<String>,
     ) -> Result<models::repos::Ref> {
-        let route = format!(
-            "/repos/{owner}/{repo}/git/refs",
-            owner = self.owner,
-            repo = self.repo,
-        );
+        let route = format!("/{}/git/refs", self.repo);
         self.crab
             .post(
                 route,
@@ -206,6 +210,31 @@ impl<'octo> RepoHandler<'octo> {
             .await
     }
 
+    /// Deletes an existing reference from the repository.
+    /// ```no_run
+    /// # async fn run() -> octocrab::Result<()> {
+    /// # let master_sha = "";
+    /// use octocrab::params::repos::Reference;
+    ///
+    /// // Deletes the "heads/temporary-branch" reference.
+    /// octocrab::instance()
+    ///     .repos("owner", "repo")
+    ///     .delete_ref(&Reference::Branch("temporary-branch".to_string()))
+    ///     .await?;
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub async fn delete_ref(&self, reference: &params::repos::Reference) -> Result<()> {
+        let route = format!(
+            "/{repo}/git/refs/{ref}",
+            repo = self.repo,
+            ref = reference.ref_url()
+        );
+        crate::map_github_error(self.crab._delete(route, None::<&()>).await?)
+            .await
+            .map(drop)
+    }
+
     /// Get repository content.
     /// ```no_run
     /// # async fn run() -> octocrab::Result<()> {
@@ -259,10 +288,12 @@ impl<'octo> RepoHandler<'octo> {
     ///     .commiter(CommitAuthor {
     ///         name: "Octocat".to_string(),
     ///         email: "octocat@github.com".to_string(),
+    ///         date: None,
     ///     })
     ///     .author(CommitAuthor {
     ///         name: "Ferris".to_string(),
     ///         email: "ferris@rust-lang.org".to_string(),
+    ///         date: None,
     ///     })
     ///     .send()
     ///     .await?;
@@ -285,6 +316,17 @@ impl<'octo> RepoHandler<'octo> {
         )
     }
 
+    /// Update an existing file.
+    ///
+    /// - `path`: the path of the updated file.
+    /// - `message`: the message of the commit used to update the file
+    /// - `content`: the updated contents of the file (base64 encoding is done
+    ///   automatically).
+    /// - `sha`: the blob SHA of the file being updated. This can be obtained
+    ///   using the [RepoHandler::get_content] function.
+    ///
+    /// [GitHub API documentation](https://docs.github.com/en/rest/repos/contents?apiVersion=2022-11-28#create-or-update-file-contents)
+    ///
     /// ```no_run
     /// # async fn run() -> octocrab::Result<()> {
     /// # let blob_sha = "";
@@ -303,10 +345,12 @@ impl<'octo> RepoHandler<'octo> {
     ///     .commiter(CommitAuthor {
     ///         name: "Octocat".to_string(),
     ///         email: "octocat@github.com".to_string(),
+    ///         date: None,
     ///     })
     ///     .author(CommitAuthor {
     ///         name: "Ferris".to_string(),
     ///         email: "ferris@rust-lang.org".to_string(),
+    ///         date: None,
     ///     })
     ///     .send()
     ///     .await?;
@@ -348,10 +392,12 @@ impl<'octo> RepoHandler<'octo> {
     ///     .commiter(CommitAuthor {
     ///         name: "Octocat".to_string(),
     ///         email: "octocat@github.com".to_string(),
+    ///         date: None,
     ///     })
     ///     .author(CommitAuthor {
     ///         name: "Ferris".to_string(),
     ///         email: "ferris@rust-lang.org".to_string(),
+    ///         date: None,
     ///     })
     ///     .send()
     ///     .await?;
@@ -426,6 +472,17 @@ impl<'octo> RepoHandler<'octo> {
         ListCollaboratorsBuilder::new(self)
     }
 
+    /// List contributors from a repository.
+    /// ```no_run
+    /// # async fn run() -> octocrab::Result<()> {
+    /// let contributors = octocrab::instance().repos("owner", "repo").list_contributors().send().await?;
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub fn list_contributors(&self) -> ListContributorsBuilder<'_, '_> {
+        ListContributorsBuilder::new(self)
+    }
+
     /// List star_gazers from a repository.
     /// ```no_run
     /// # async fn run() -> octocrab::Result<()> {
@@ -437,6 +494,30 @@ impl<'octo> RepoHandler<'octo> {
         ListStarGazersBuilder::new(self)
     }
 
+    /// Lists languages for the specified repository.
+    /// The value shown for each language is the number of bytes of code written in that language.
+    ///
+    /// ```no_run
+    /// # async fn run() -> octocrab::Result<()> {
+    ///
+    /// // Get the languages used in the repository
+    /// let languages = octocrab::instance()
+    ///     .repos("owner", "repo")
+    ///     .list_languages()
+    ///     .await?;
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub async fn list_languages(&self) -> Result<models::repos::Languages> {
+        let route = format!("/{}/languages", self.repo);
+        self.crab.get(route, None::<&()>).await
+    }
+
+    /// Creates a `ReleaseAssetsHandler` for the specified repository.
+    pub fn release_assets(&self) -> release_assets::ReleaseAssetsHandler<'_, '_> {
+        release_assets::ReleaseAssetsHandler::new(self)
+    }
+
     /// Creates a `ReleasesHandler` for the specified repository.
     pub fn releases(&self) -> releases::ReleasesHandler<'_, '_> {
         releases::ReleasesHandler::new(self)
@@ -488,6 +569,44 @@ impl<'octo> RepoHandler<'octo> {
         events::ListRepoEventsBuilder::new(self)
     }
 
+    /// Creates a new webhook for the specified repository.
+    ///
+    /// # Notes
+    /// Only authorized users or apps can modify repository webhooks.
+    ///
+    /// # Examples
+    /// ```no_run
+    /// # async fn run() -> octocrab::Result<()> {
+    /// # let octocrab = octocrab::Octocrab::default();
+    /// use octocrab::models::hooks::{Hook, Config as HookConfig, ContentType as HookContentType};
+    ///
+    /// let config = HookConfig {
+    ///   url: "https://example.com".to_string(),
+    ///   content_type: Some(HookContentType::Json),
+    ///   insecure_ssl: None,
+    ///   secret: None
+    /// };
+    ///
+    /// let hook = Hook {
+    ///   name: "web".to_string(),
+    ///   config,
+    ///   ..Hook::default()
+    /// };
+    ///
+    /// let hook = octocrab.repos("owner", "repo").create_hook(hook).await?;
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub async fn create_hook(
+        &self,
+        hook: crate::models::hooks::Hook,
+    ) -> crate::Result<crate::models::hooks::Hook> {
+        let route = format!("/{}/hooks", self.repo);
+        let res = self.crab.post(route, Some(&hook)).await?;
+
+        Ok(res)
+    }
+
     /// Gets the combined status for the specified reference.
     /// ```no_run
     /// # async fn run() -> octocrab::Result<()> {
@@ -505,8 +624,7 @@ impl<'octo> RepoHandler<'octo> {
         reference: &params::repos::Reference,
     ) -> Result<models::CombinedStatus> {
         let route = format!(
-            "/repos/{owner}/{repo}/commits/{reference}/status",
-            owner = self.owner,
+            "/{repo}/commits/{reference}/status",
             repo = self.repo,
             reference = reference.ref_url(),
         );
@@ -537,10 +655,9 @@ impl<'octo> RepoHandler<'octo> {
         self,
         reference: impl Into<params::repos::Commitish>,
         path: impl AsRef<str>,
-    ) -> Result<http::Response<hyper::Body>> {
+    ) -> Result<http::Response<BoxBody<Bytes, crate::Error>>> {
         let route = format!(
-            "/repos/{owner}/{repo}/contents/{path}",
-            owner = self.owner,
+            "/{repo}/contents/{path}",
             repo = self.repo,
             path = path.as_ref(),
         );
@@ -564,11 +681,7 @@ impl<'octo> RepoHandler<'octo> {
     /// # }
     /// ```
     pub async fn delete(self) -> Result<()> {
-        let route = format!(
-            "/repos/{owner}/{repo}",
-            owner = self.owner,
-            repo = self.repo
-        );
+        let route = format!("/{}", self.repo);
         let uri = Uri::builder()
             .path_and_query(route)
             .build()
@@ -582,10 +695,9 @@ impl<'octo> RepoHandler<'octo> {
     pub async fn download_tarball(
         &self,
         reference: impl Into<params::repos::Commitish>,
-    ) -> Result<http::Response<hyper::Body>> {
+    ) -> Result<http::Response<BoxBody<Bytes, crate::Error>>> {
         let route = format!(
-            "/repos/{owner}/{repo}/tarball/{reference}",
-            owner = self.owner,
+            "/{repo}/tarball/{reference}",
             repo = self.repo,
             reference = reference.into(),
         );
@@ -601,8 +713,7 @@ impl<'octo> RepoHandler<'octo> {
     /// Check if a user is a repository collaborator
     pub async fn is_collaborator(&self, username: impl AsRef<str>) -> Result<bool> {
         let route = format!(
-            "/repos/{owner}/{repo}/collaborators/{username}",
-            owner = self.owner,
+            "/{repo}/collaborators/{username}",
             repo = self.repo,
             username = username.as_ref(),
         );
@@ -640,4 +751,116 @@ impl<'octo> RepoHandler<'octo> {
     pub fn secrets(&self) -> RepoSecretsHandler<'_> {
         RepoSecretsHandler::new(self)
     }
+
+    /// Handle dependabot alerts on the repository
+    pub fn dependabot(&self) -> RepoDependabotAlertsHandler<'_> {
+        RepoDependabotAlertsHandler::new(self)
+    }
+
+    /// Handle secrets scanning alerts on the repository
+    pub fn secrets_scanning(&self) -> RepoSecretScanningAlertsHandler<'_> {
+        RepoSecretScanningAlertsHandler::new(self)
+    }
+
+    /// Creates a new Git commit object.
+    /// See https://docs.github.com/en/rest/git/commits?apiVersion=2022-11-28#create-a-commit
+    /// ```no_run
+    /// # use octocrab::models::commits::GitCommitObject;
+    /// use octocrab::models::repos::CommitAuthor;
+    ///  async fn run() -> octocrab::Result<(GitCommitObject)> {
+    ///
+    /// let git_commit_object = octocrab::instance()
+    ///     .repos("owner", "repo")
+    ///     .create_git_commit_object("message", "tree")
+    ///     .signature("signature")
+    ///     .author(CommitAuthor{
+    ///             name: "name".to_owned(),
+    ///             email: "email".to_owned(),
+    ///             date: None
+    ///         })
+    ///     .send()
+    ///     .await;
+    /// #   git_commit_object
+    /// # }
+    /// ```
+    pub fn create_git_commit_object(
+        &self,
+        message: impl Into<String>,
+        tree: impl Into<String>,
+    ) -> CreateGitCommitObjectBuilder<'_, '_> {
+        CreateGitCommitObjectBuilder::new(
+            self,
+            self.repo.clone(),
+            message.into().to_owned(),
+            tree.into().to_owned(),
+        )
+    }
+}
+
+#[derive(serde::Serialize)]
+pub struct CreateGitCommitObjectBuilder<'octo, 'req> {
+    #[serde(skip)]
+    handler: &'octo RepoHandler<'req>,
+    // According to [API reference](https://docs.github.com/en/rest/git/commits?apiVersion=2022-11-28#create-a-commit), repo in body is not required.
+    #[serde(skip)]
+    repo: RepoRef,
+    message: String,
+    tree: String,
+    parents: Vec<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    author: Option<repos::CommitAuthor>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    committer: Option<repos::CommitAuthor>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    signature: Option<String>,
+}
+
+impl<'octo, 'req> CreateGitCommitObjectBuilder<'octo, 'req> {
+    pub(crate) fn new(
+        handler: &'octo RepoHandler<'req>,
+        repo: RepoRef,
+        message: String,
+        tree: String,
+    ) -> Self {
+        Self {
+            handler,
+            repo,
+            message,
+            tree,
+            parents: Vec::new(),
+            author: None,
+            committer: None,
+            signature: None,
+        }
+    }
+
+    /// The author of the commit.
+    pub fn author(mut self, author: impl Into<repos::CommitAuthor>) -> Self {
+        self.author = Some(author.into());
+        self
+    }
+
+    /// The committer of the commit.
+    pub fn committer(mut self, committer: impl Into<repos::CommitAuthor>) -> Self {
+        self.committer = Some(committer.into());
+        self
+    }
+
+    /// The signature of the commit.
+    pub fn signature(mut self, signature: impl Into<String>) -> Self {
+        self.signature = Some(signature.into());
+        self
+    }
+
+    /// The parents of the commit.
+    pub fn parents(mut self, parents: Vec<String>) -> Self {
+        self.parents = parents;
+        self
+    }
+
+    /// Sends the request
+    pub async fn send(&self) -> Result<GitCommitObject> {
+        let route = format!("/{}/git/commits", self.repo);
+        self.handler.crab.post(route, Some(&self)).await
+    }
 }
