@@ -36,12 +36,7 @@ impl<'octo, 'r> ListPullsBuilder<'octo, 'r> {
 
     /// Sends the actual request.
     pub async fn send(self) -> crate::Result<crate::Page<crate::models::pulls::PullRequest>> {
-        let route = format!(
-            "/repos/{owner}/{repo}/commits/{sha}/pulls",
-            owner = self.handler.owner,
-            repo = self.handler.repo,
-            sha = self.sha,
-        );
+        let route = format!("/{}/commits/{sha}/pulls", self.handler.repo, sha = self.sha,);
         self.handler.crab.get(route, Some(&self)).await
     }
 }
