@@ -1,17 +1,19 @@
-use crate::models::AssetId;
-
 use super::*;
+use crate::error::{UriParseError, UriParseSnafu};
+use crate::from_response::FromResponse;
+use crate::models::repos::Asset;
+use std::convert::TryInto;
 
 /// Handler for GitHub's releases API.
 ///
 /// Created with [`RepoHandler::releases`].
 pub struct ReleasesHandler<'octo, 'r> {
-    parent: &'r RepoHandler<'octo>,
+    handler: &'r RepoHandler<'octo>,
 }
 
 impl<'octo, 'r> ReleasesHandler<'octo, 'r> {
     pub(crate) fn new(parent: &'r RepoHandler<'octo>) -> Self {
-        Self { parent }
+        Self { handler: parent }
     }
 
     /// Creates a new [`ListReleasesBuilder`] that can be configured to filter
@@ -96,15 +98,9 @@ impl<'octo, 'r> ReleasesHandler<'octo, 'r> {
     /// # Ok(())
     /// # }
     /// ```
-    pub async fn get_asset(&self, asset_id: AssetId) -> crate::Result<models::repos::Asset> {
-        let route = format!(
-            "/repos/{owner}/{repo}/releases/assets/{asset_id}",
-            owner = self.parent.owner,
-            repo = self.parent.repo,
-            asset_id = asset_id,
-        );
-
-        self.parent.crab.get(route, None::<&()>).await
+    #[deprecated(note = "use repos::ReleaseAssetsHandler::get instead")]
+    pub async fn get_asset(&self, asset_id: u64) -> crate::Result<models::repos::Asset> {
+        self.handler.release_assets().get(asset_id).await
     }
 
     /// Gets the latest release.
@@ -119,13 +115,9 @@ impl<'octo, 'r> ReleasesHandler<'octo, 'r> {
     /// # }
     /// ```
     pub async fn get_latest(&self) -> crate::Result<models::repos::Release> {
-        let route = format!(
-            "/repos/{owner}/{repo}/releases/latest",
-            owner = self.parent.owner,
-            repo = self.parent.repo,
-        );
+        let route = format!("/{}/releases/latest", self.handler.repo,);
 
-        self.parent.crab.get(route, None::<&()>).await
+        self.handler.crab.get(route, None::<&()>).await
     }
 
     /// Gets the release using its tag.
@@ -140,14 +132,92 @@ impl<'octo, 'r> ReleasesHandler<'octo, 'r> {
     /// # }
     /// ```
     pub async fn get_by_tag(&self, tag: &str) -> crate::Result<models::repos::Release> {
-        let route = format!(
-            "/repos/{owner}/{repo}/releases/tags/{tag}",
-            owner = self.parent.owner,
-            repo = self.parent.repo,
-            tag = tag,
-        );
+        let route = format!("/{}/releases/tags/{tag}", self.handler.repo, tag = tag,);
 
-        self.parent.crab.get(route, None::<&()>).await
+        self.handler.crab.get(route, None::<&()>).await
+    }
+
+    /// Gets the release using its id.
+    /// ```no_run
+    /// # async fn run() -> octocrab::Result<()> {
+    /// let release = octocrab::instance()
+    ///     .repos("owner", "repo")
+    ///     .releases()
+    ///     .get(3)
+    ///     .await?;
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub async fn get(&self, number: u64) -> Result<models::repos::Release> {
+        let route = format!("/{}/releases/{number}", self.handler.repo, number = number,);
+
+        self.handler.crab.get(route, None::<&()>).await
+    }
+
+    /// Generates [`crate::models::repos::ReleaseNotes`] which describe
+    /// a [`crate::models::repos::Release`]
+    /// ```no_run
+    /// # async fn run() -> octocrab::Result<()> {
+    /// let release_notes = octocrab::instance()
+    ///     .repos("owner", "repo")
+    ///     .releases()
+    ///     .generate_release_notes("0.1.0")
+    ///     .send()
+    ///     .await?;
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub fn generate_release_notes<'tag_name>(
+        &self,
+        tag_name: &'tag_name (impl AsRef<str> + ?Sized),
+    ) -> GenerateReleaseNotesBuilder<'_, '_, '_, 'tag_name, '_, '_, '_> {
+        GenerateReleaseNotesBuilder::new(self, tag_name.as_ref())
+    }
+
+    /// Upload an [`crate::models::repos::Asset`] associated with
+    /// a [`crate::models::repos::Release`]
+    /// ```no_run
+    /// use bytes::Bytes;
+    /// # async fn run() -> octocrab::Result<()> {
+    /// let file_data: Bytes = Bytes::from("some_data");
+    /// let asset = octocrab::instance()
+    ///     .repos("owner", "repo")
+    ///     .releases()
+    ///     .upload_asset(1, "my_asset.tar.gz", file_data)
+    ///     .label("My Awesome Asset")
+    ///     .send()
+    ///     .await?;
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub fn upload_asset<'asset_name>(
+        &self,
+        id: u64,
+        asset_name: &'asset_name (impl AsRef<str> + ?Sized),
+        body: Bytes,
+    ) -> UploadAssetBuilder<'_, '_, '_, 'asset_name, '_> {
+        UploadAssetBuilder::new(self, id, asset_name.as_ref(), body)
+    }
+
+    /// Creates a new [`ListReleaseAssetsBuilder`] that can be configured to filter
+    /// listing release assetss.
+    /// ```no_run
+    /// # async fn run() -> octocrab::Result<()> {
+    /// # let octocrab = octocrab::Octocrab::default();
+    /// let page = octocrab.repos("owner", "repo")
+    ///     .releases()
+    ///     .assets(1)
+    ///     // Optional Parameters
+    ///     .per_page(100)
+    ///     .page(5u32)
+    ///     // Send the request
+    ///     .send()
+    ///     .await?;
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub fn assets(&self, release_id: u64) -> ListReleaseAssetsBuilder<'_, '_, '_> {
+        ListReleaseAssetsBuilder::new(self, release_id)
     }
 
     /// Streams the binary contents of an asset.
@@ -169,36 +239,30 @@ impl<'octo, 'r> ReleasesHandler<'octo, 'r> {
     /// ```
     #[cfg(feature = "stream")]
     #[cfg_attr(docsrs, doc(cfg(feature = "stream")))]
+    #[deprecated(note = "use repos::ReleaseAssetsHandler::stream instead")]
     pub async fn stream_asset(
         &self,
-        asset_id: AssetId,
+        asset_id: u64,
     ) -> crate::Result<impl futures_core::Stream<Item = crate::Result<bytes::Bytes>>> {
-        use futures_util::TryStreamExt;
-        use snafu::GenerateImplicitData;
+        self.handler.release_assets().stream(asset_id).await
+    }
 
-        let route = format!(
-            "/repos/{owner}/{repo}/releases/assets/{asset_id}",
-            owner = self.parent.owner,
-            repo = self.parent.repo,
-            asset_id = asset_id,
-        );
+    /// Delete a release using its id.
+    /// ```no_run
+    /// # async fn run() -> octocrab::Result<()> {
+    /// let release = octocrab::instance()
+    ///     .repos("owner", "repo")
+    ///     .releases()
+    ///     .delete(3)
+    ///     .await?;
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub async fn delete(&self, id: u64) -> Result<()> {
+        let route = format!("/{}/releases/{id}", self.handler.repo, id = id,);
 
-        let uri = Uri::builder()
-            .path_and_query(route)
-            .build()
-            .context(HttpSnafu)?;
-        let builder = Builder::new()
-            .method(http::Method::GET)
-            .uri(uri)
-            .header(http::header::ACCEPT, "application/octet-stream");
-        let request = self.parent.crab.build_request(builder, None::<&()>)?;
-        let response = self.parent.crab.execute(request).await?;
-        Ok(response
-            .into_body()
-            .map_err(|source| crate::error::Error::Hyper {
-                source,
-                backtrace: snafu::Backtrace::generate(),
-            }))
+        self.handler.crab._delete(route, None::<&()>).await?;
+        Ok(())
     }
 }
 
@@ -238,12 +302,8 @@ impl<'octo, 'r1, 'r2> ListReleasesBuilder<'octo, 'r1, 'r2> {
 
     /// Sends the actual request.
     pub async fn send(self) -> crate::Result<crate::Page<crate::models::repos::Release>> {
-        let route = format!(
-            "/repos/{owner}/{repo}/releases",
-            owner = self.handler.parent.owner,
-            repo = self.handler.parent.repo
-        );
-        self.handler.parent.crab.get(route, Some(&self)).await
+        let route = format!("/{}/releases", self.handler.handler.repo);
+        self.handler.handler.crab.get(route, Some(&self)).await
     }
 }
 
@@ -356,12 +416,8 @@ impl<'octo, 'repos, 'handler, 'tag_name, 'target_commitish, 'name, 'body>
 
     /// Sends the actual request.
     pub async fn send(self) -> crate::Result<crate::models::repos::Release> {
-        let route = format!(
-            "/repos/{owner}/{repo}/releases",
-            owner = self.handler.parent.owner,
-            repo = self.handler.parent.repo
-        );
-        self.handler.parent.crab.post(route, Some(&self)).await
+        let route = format!("/{}/releases", self.handler.handler.repo);
+        self.handler.handler.crab.post(route, Some(&self)).await
     }
 }
 
@@ -386,6 +442,8 @@ pub struct UpdateReleaseBuilder<'octo, 'repos, 'handler, 'tag_name, 'target_comm
     draft: Option<bool>,
     #[serde(skip_serializing_if = "Option::is_none")]
     prerelease: Option<bool>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    make_latest: Option<MakeLatest>,
 }
 
 impl<'octo, 'repos, 'handler, 'tag_name, 'target_commitish, 'name, 'body>
@@ -401,6 +459,7 @@ impl<'octo, 'repos, 'handler, 'tag_name, 'target_commitish, 'name, 'body>
             body: None,
             draft: None,
             prerelease: None,
+            make_latest: None,
         }
     }
 
@@ -446,14 +505,231 @@ impl<'octo, 'repos, 'handler, 'tag_name, 'target_commitish, 'name, 'body>
         self
     }
 
+    /// Specifies whether this release should be set as the latest release for the repository.
+    /// Drafts and prereleases cannot be set as latest.
+    /// [`MakeLatest::Legacy`] specifies that the latest release should be determined based on the release creation date and higher semantic version.
+    pub fn make_latest(mut self, make_latest: MakeLatest) -> Self {
+        self.make_latest = Some(make_latest);
+        self
+    }
+
     /// Sends the actual request.
     pub async fn send(self) -> crate::Result<crate::models::repos::Release> {
         let route = format!(
-            "/repos/{owner}/{repo}/releases/{release_id}",
-            owner = self.handler.parent.owner,
-            repo = self.handler.parent.repo,
+            "/{repo}/releases/{release_id}",
+            repo = self.handler.handler.repo,
             release_id = self.release_id,
         );
-        self.handler.parent.crab.patch(route, Some(&self)).await
+        self.handler.handler.crab.patch(route, Some(&self)).await
+    }
+}
+
+/// A builder pattern struct for updating releases.
+///
+/// created by [`ReleasesHandler::generate_release_notes`].
+#[derive(serde::Serialize)]
+pub struct GenerateReleaseNotesBuilder<
+    'octo,
+    'repos,
+    'handler,
+    'tag_name,
+    'previous_tag_name,
+    'target_commitish,
+    'configuration_file_path,
+> {
+    #[serde(skip)]
+    handler: &'handler ReleasesHandler<'octo, 'repos>,
+    tag_name: &'tag_name str,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    previous_tag_name: Option<&'previous_tag_name str>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    target_commitish: Option<&'target_commitish str>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    configuration_file_path: Option<&'configuration_file_path str>,
+}
+
+impl<
+        'octo,
+        'repos,
+        'handler,
+        'tag_name,
+        'previous_tag_name,
+        'target_commitish,
+        'configuration_file_path,
+    >
+    GenerateReleaseNotesBuilder<
+        'octo,
+        'repos,
+        'handler,
+        'tag_name,
+        'previous_tag_name,
+        'target_commitish,
+        'configuration_file_path,
+    >
+{
+    pub(crate) fn new(
+        handler: &'handler ReleasesHandler<'octo, 'repos>,
+        tag_name: &'tag_name str,
+    ) -> Self {
+        Self {
+            handler,
+            tag_name,
+            previous_tag_name: None,
+            target_commitish: None,
+            configuration_file_path: None,
+        }
+    }
+
+    /// The tag which is used as a starting point for the release notes.
+    pub fn previous_tag_name(
+        mut self,
+        previous_tag_name: &'previous_tag_name (impl AsRef<str> + ?Sized),
+    ) -> Self {
+        self.previous_tag_name = Some(previous_tag_name.as_ref());
+        self
+    }
+
+    /// Specifies the commitish value that determines where the Git tag is
+    /// created from. Can be any branch or commit SHA.
+    /// Unused if the Git [`GenerateReleaseNotesBuilder::tag_name`] exists.
+    pub fn target_commitish(
+        mut self,
+        target_commitish: &'target_commitish (impl AsRef<str> + ?Sized),
+    ) -> Self {
+        self.target_commitish = Some(target_commitish.as_ref());
+        self
+    }
+
+    /// A file path within the repository which contains the configuration settings
+    /// for generating release notes.
+    pub fn configuration_file_path(
+        mut self,
+        configuration_file_path: &'configuration_file_path (impl AsRef<str> + ?Sized),
+    ) -> Self {
+        self.configuration_file_path = Some(configuration_file_path.as_ref());
+        self
+    }
+
+    /// Sends the actual request.
+    pub async fn send(self) -> crate::Result<crate::models::repos::ReleaseNotes> {
+        let route = format!("/{}/releases/generate-notes", self.handler.handler.repo,);
+
+        let result: Result<crate::models::repos::ReleaseNotes> =
+            self.handler.handler.crab.post(route, Some(&self)).await;
+        result
+    }
+}
+
+// A builder pattern struct for listing release assets.
+///
+/// created by [`ReleasesHandler::assets`]
+#[derive(serde::Serialize)]
+pub struct ListReleaseAssetsBuilder<'octo, 'r1, 'r2> {
+    #[serde(skip)]
+    handler: &'r2 ReleasesHandler<'octo, 'r1>,
+    #[serde(skip)]
+    release_id: u64,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    per_page: Option<u8>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    page: Option<u32>,
+}
+
+impl<'octo, 'r1, 'r2> ListReleaseAssetsBuilder<'octo, 'r1, 'r2> {
+    pub(crate) fn new(handler: &'r2 ReleasesHandler<'octo, 'r1>, release_id: u64) -> Self {
+        Self {
+            handler,
+            release_id,
+            per_page: None,
+            page: None,
+        }
+    }
+
+    /// Results per page (max 100).
+    pub fn per_page(mut self, per_page: impl Into<u8>) -> Self {
+        self.per_page = Some(per_page.into());
+        self
+    }
+
+    /// Page number of the results to fetch.
+    pub fn page(mut self, page: impl Into<u32>) -> Self {
+        self.page = Some(page.into());
+        self
+    }
+
+    /// Sends the actual request.
+    pub async fn send(self) -> crate::Result<crate::Page<crate::models::repos::Asset>> {
+        let route = format!(
+            "/{repo}/releases/{release_id}/assets",
+            repo = self.handler.handler.repo,
+            release_id = self.release_id,
+        );
+        self.handler.handler.crab.get(route, Some(&self)).await
+    }
+}
+
+/// A builder pattern struct for updating release assets.
+///
+/// created by [`ReleasesHandler::upload_asset`].
+pub struct UploadAssetBuilder<'octo, 'repos, 'handler, 'name, 'label> {
+    handler: &'handler ReleasesHandler<'octo, 'repos>,
+    release_id: u64,
+    name: &'name str,
+    body: Bytes,
+    label: Option<&'label str>,
+}
+
+impl<'octo, 'repos, 'handler, 'name, 'label>
+    UploadAssetBuilder<'octo, 'repos, 'handler, 'name, 'label>
+{
+    pub(crate) fn new(
+        handler: &'handler ReleasesHandler<'octo, 'repos>,
+        release_id: u64,
+        name: &'name str,
+        body: Bytes,
+    ) -> Self {
+        Self {
+            handler,
+            release_id,
+            name,
+            body,
+            label: None,
+        }
+    }
+
+    /// The asset label
+    pub fn label(mut self, label: &'label (impl AsRef<str> + ?Sized)) -> Self {
+        self.label = Some(label.as_ref());
+        self
+    }
+
+    /// Sends the actual request.
+    pub async fn send(self) -> crate::Result<Asset> {
+        // the url could be constructed without fetching the release, but if the user has no access to the release
+        // then he will not have access to upload to it.
+        let release = self.handler.get(self.release_id).await?;
+
+        let mut base_uri = format!(
+            "{}?name={}",
+            release.upload_url.replace("{?name,label}", ""),
+            self.name
+        );
+        if let Some(label) = self.label {
+            base_uri = format!("{}&label={}", base_uri, label);
+        }
+
+        let url: Uri = base_uri
+            .try_into()
+            .map_err(|_| UriParseError {})
+            .context(UriParseSnafu)?;
+        let request = Builder::new()
+            .method(http::Method::POST)
+            .uri(url)
+            .header(http::header::CONTENT_TYPE, "application/octet-stream")
+            .header(http::header::CONTENT_LENGTH, self.body.len())
+            .body(self.body)
+            .context(HttpSnafu)?;
+        let response = self.handler.handler.crab.execute(request).await?;
+        Asset::from_response(crate::map_github_error(response).await?).await
     }
 }
