@@ -179,8 +179,10 @@
 //! };
 //! ```
 #![cfg_attr(test, recursion_limit = "512")]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 
 mod api;
+mod body;
 mod error;
 mod from_response;
 mod page;
@@ -190,26 +192,31 @@ pub mod etag;
 pub mod models;
 pub mod params;
 pub mod service;
-use crate::service::body::BodyStreamExt;
 
+use api::repos::RepoRef;
+use api::users::UserRef;
+use body::OctoBody;
 use chrono::{DateTime, Utc};
 use http::{HeaderMap, HeaderValue, Method, Uri};
+use http_body_util::combinators::BoxBody;
+use http_body_util::BodyExt;
+use service::middleware::auth_header::AuthHeaderLayer;
 use std::convert::{Infallible, TryInto};
 use std::fmt;
+use std::future::Future;
 use std::io::Write;
 use std::marker::PhantomData;
+use std::pin::Pin;
 use std::str::FromStr;
 use std::sync::{Arc, RwLock};
-use std::time::Duration;
+use web_time::Duration;
 
 use http::{header::HeaderName, StatusCode};
-#[cfg(all(not(feature = "opentls"), not(feature = "rustls")))]
-use hyper::client::HttpConnector;
-use hyper::{body, Body, Request, Response};
+use hyper::{Request, Response};
 
 use once_cell::sync::Lazy;
 use secrecy::{ExposeSecret, SecretString};
-use serde::Serialize;
+use serde::{Deserialize, Serialize};
 use snafu::*;
 use tower::{buffer::Buffer, util::BoxService, BoxError, Layer, Service, ServiceExt};
 
@@ -226,10 +233,7 @@ use hyper_rustls::HttpsConnectorBuilder;
 use tower::retry::{Retry, RetryLayer};
 
 #[cfg(feature = "timeout")]
-use {
-    hyper_timeout::TimeoutConnector,
-    tokio::io::{AsyncRead, AsyncWrite},
-};
+use hyper_timeout::TimeoutConnector;
 
 use tower_http::{classify::ServerErrorsFailureClass, map_response_body::MapResponseBodyLayer};
 
@@ -247,13 +251,13 @@ use crate::service::middleware::extra_headers::ExtraHeadersLayer;
 #[cfg(feature = "retry")]
 use crate::service::middleware::retry::RetryConfig;
 
-use crate::api::users;
+use crate::api::{code_scannings, users};
 use auth::{AppAuth, Auth};
-use models::{AppId, InstallationId, InstallationToken};
+use models::{AppId, InstallationId, InstallationToken, RepositoryId, UserId};
 
 pub use self::{
     api::{
-        actions, activity, apps, checks, commits, current, events, gists, gitignore, issues,
+        actions, activity, apps, checks, commits, current, events, gists, gitignore, hooks, issues,
         licenses, markdown, orgs, projects, pulls, ratelimit, repos, search, teams, workflows,
     },
     error::{Error, GitHubError},
@@ -265,7 +269,9 @@ pub use self::{
 pub type Result<T, E = error::Error> = std::result::Result<T, E>;
 
 const GITHUB_BASE_URI: &str = "https://api.github.com";
+const GITHUB_BASE_UPLOAD_URI: &str = "https://uploads.github.com";
 
+#[cfg(feature = "default-client")]
 static STATIC_INSTANCE: Lazy<arc_swap::ArcSwap<Octocrab>> =
     Lazy::new(|| arc_swap::ArcSwap::from_pointee(Octocrab::default()));
 
@@ -295,25 +301,37 @@ pub fn format_media_type(media_type: impl AsRef<str>) -> String {
     format!("application/vnd.github.v3.{media_type}{json_suffix}")
 }
 
+#[derive(Debug, Deserialize)]
+struct GitHubErrorBody {
+    pub documentation_url: Option<String>,
+    pub errors: Option<Vec<serde_json::Value>>,
+    pub message: String,
+}
+
 /// Maps a GitHub error response into and `Err()` variant if the status is
 /// not a success.
 pub async fn map_github_error(
-    response: http::Response<hyper::Body>,
-) -> Result<http::Response<hyper::Body>> {
+    response: http::Response<BoxBody<Bytes, crate::Error>>,
+) -> Result<http::Response<BoxBody<Bytes, crate::Error>>> {
     if response.status().is_success() {
         Ok(response)
     } else {
-        let b: error::GitHubError = serde_json::from_slice(
-            body::to_bytes(response.into_body())
-                .await
-                .context(error::HyperSnafu)?
-                .as_ref(),
-        )
-        .context(error::SerdeSnafu)?;
+        let (parts, body) = response.into_parts();
+        let GitHubErrorBody {
+            documentation_url,
+            errors,
+            message,
+        } = serde_json::from_slice(body.collect().await?.to_bytes().as_ref())
+            .context(error::SerdeSnafu)?;
 
         Err(error::Error::GitHub {
-            source: b,
-            backtrace: Backtrace::generate(),
+            source: Box::new(GitHubError {
+                status_code: parts.status,
+                documentation_url,
+                errors,
+                message,
+            }),
+            backtrace: Backtrace::capture(),
         })
     }
 }
@@ -327,6 +345,8 @@ pub async fn map_github_error(
 /// # Ok(())
 /// # }
 /// ```
+#[cfg(feature = "default-client")]
+#[cfg_attr(docsrs, doc(cfg(feature = "default-client")))]
 pub fn initialise(crab: Octocrab) -> Arc<Octocrab> {
     STATIC_INSTANCE.swap(Arc::from(crab))
 }
@@ -339,12 +359,17 @@ pub fn initialise(crab: Octocrab) -> Arc<Octocrab> {
 /// let octocrab = octocrab::instance();
 /// }
 /// ```
+#[cfg(feature = "default-client")]
+#[cfg_attr(docsrs, doc(cfg(feature = "default-client")))]
 pub fn instance() -> Arc<Octocrab> {
     STATIC_INSTANCE.load().clone()
 }
 
+type Executor = Box<dyn Fn(Pin<Box<dyn Future<Output = ()>>>)>;
+
 /// A builder struct for `Octocrab`, allowing you to configure the client, such
 /// as using GitHub previews, the github instance, authentication, etc.
+///
 /// ```
 /// # #[tokio::main]
 /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
@@ -355,16 +380,14 @@ pub fn instance() -> Arc<Octocrab> {
 /// # Ok(())
 /// # }
 /// ```
-
-//Typed builder, thanks to https://www.greyblake.com/blog/builder-with-typestate-in-rust/ for explaining
-
-/// A builder struct for `Octocrab`.
+///
 /// OctocrabBuilder can be extended with a custom config, see [DefaultOctocrabBuilderConfig] for an example
 pub struct OctocrabBuilder<Svc, Config, Auth, LayerReady> {
     service: Svc,
     auth: Auth,
     config: Config,
     _layer_ready: PhantomData<LayerReady>,
+    executor: Option<Executor>,
 }
 
 //Indicates weather the builder supports config
@@ -387,6 +410,7 @@ impl OctocrabBuilder<NoSvc, NoConfig, NoAuth, NotLayerReady> {
             auth: NoAuth {},
             config: NoConfig {},
             _layer_ready: PhantomData,
+            executor: None,
         }
     }
 }
@@ -404,13 +428,36 @@ impl<Config, Auth> OctocrabBuilder<NoSvc, Config, Auth, NotLayerReady> {
             auth: self.auth,
             config: self.config,
             _layer_ready: PhantomData,
+            executor: None,
         }
     }
 }
 
 impl<Svc, Config, Auth, B> OctocrabBuilder<Svc, Config, Auth, LayerReady>
 where
-    Svc: Service<Request<String>, Response = Response<B>> + Send + 'static,
+    Svc: Service<Request<OctoBody>, Response = Response<B>> + Send + 'static,
+    Svc::Future: Send + 'static,
+    Svc::Error: Into<BoxError>,
+    B: http_body::Body<Data = bytes::Bytes> + Send + 'static,
+    B::Error: Into<BoxError>,
+{
+    pub fn with_executor(
+        self,
+        executor: Executor,
+    ) -> OctocrabBuilder<Svc, Config, Auth, LayerReady> {
+        OctocrabBuilder {
+            service: self.service,
+            auth: self.auth,
+            config: self.config,
+            _layer_ready: PhantomData,
+            executor: Some(executor),
+        }
+    }
+}
+
+impl<Svc, Config, Auth, B> OctocrabBuilder<Svc, Config, Auth, LayerReady>
+where
+    Svc: Service<Request<OctoBody>, Response = Response<B>> + Send + 'static,
     Svc::Future: Send + 'static,
     Svc::Error: Into<BoxError>,
     B: http_body::Body<Data = bytes::Bytes> + Send + 'static,
@@ -425,12 +472,14 @@ where
             service: stack,
             auth,
             config,
+            executor,
             ..
         } = self;
         OctocrabBuilder {
             service: layer.layer(stack),
             auth,
             config,
+            executor,
             _layer_ready: PhantomData,
         }
     }
@@ -447,6 +496,7 @@ impl<Svc, Auth, LayerState> OctocrabBuilder<Svc, NoConfig, Auth, LayerState> {
         OctocrabBuilder {
             service: self.service,
             auth: self.auth,
+            executor: self.executor,
             config,
             _layer_ready: PhantomData,
         }
@@ -455,15 +505,26 @@ impl<Svc, Auth, LayerState> OctocrabBuilder<Svc, NoConfig, Auth, LayerState> {
 
 impl<Svc, B, LayerState> OctocrabBuilder<Svc, NoConfig, AuthState, LayerState>
 where
-    Svc: Service<Request<String>, Response = Response<B>> + Send + 'static,
+    Svc: Service<Request<OctoBody>, Response = Response<B>> + Send + 'static,
     Svc::Future: Send + 'static,
     Svc::Error: Into<BoxError>,
-    B: http_body::Body<Data = bytes::Bytes> + Send + 'static,
+    B: http_body::Body<Data = bytes::Bytes> + Send + Sync + 'static,
     B::Error: Into<BoxError>,
 {
     /// Build a [`Client`] instance with the current [`Service`] stack.
     pub fn build(self) -> Result<Octocrab, Infallible> {
-        Ok(Octocrab::new(self.service, self.auth))
+        // Transform response body to `BoxBody<Bytes, crate::Error>` and use type erased error to avoid type parameters.
+        let service = MapResponseBodyLayer::new(|b: B| {
+            b.map_err(|e| ServiceSnafu.into_error(e.into())).boxed()
+        })
+        .layer(self.service)
+        .map_err(|e| e.into());
+
+        if let Some(executor) = self.executor {
+            return Ok(Octocrab::new_with_executor(service, self.auth, executor));
+        }
+
+        Ok(Octocrab::new(service, self.auth))
     }
 }
 
@@ -473,18 +534,45 @@ impl<Svc, Config, LayerState> OctocrabBuilder<Svc, Config, NoAuth, LayerState> {
             service: self.service,
             auth,
             config: self.config,
+            executor: self.executor,
             _layer_ready: PhantomData,
         }
     }
 }
 
 impl OctocrabBuilder<NoSvc, DefaultOctocrabBuilderConfig, NoAuth, NotLayerReady> {
+    /// Set the retry configuration
     #[cfg(feature = "retry")]
-    pub fn add_retry_config(&mut self, retry_config: RetryConfig) -> &mut Self {
+    #[cfg_attr(docsrs, doc(cfg(feature = "retry")))]
+    pub fn add_retry_config(mut self, retry_config: RetryConfig) -> Self {
         self.config.retry_config = retry_config;
         self
     }
 
+    /// Set the connect timeout.
+    #[cfg(feature = "timeout")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "timeout")))]
+    pub fn set_connect_timeout(mut self, timeout: Option<Duration>) -> Self {
+        self.config.connect_timeout = timeout;
+        self
+    }
+
+    /// Set the read timeout.
+    #[cfg(feature = "timeout")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "timeout")))]
+    pub fn set_read_timeout(mut self, timeout: Option<Duration>) -> Self {
+        self.config.read_timeout = timeout;
+        self
+    }
+
+    /// Set the write timeout.
+    #[cfg(feature = "timeout")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "timeout")))]
+    pub fn set_write_timeout(mut self, timeout: Option<Duration>) -> Self {
+        self.config.write_timeout = timeout;
+        self
+    }
+
     /// Enable a GitHub preview.
     pub fn add_preview(mut self, preview: &'static str) -> Self {
         self.config.previews.push(preview);
@@ -498,8 +586,8 @@ impl OctocrabBuilder<NoSvc, DefaultOctocrabBuilderConfig, NoAuth, NotLayerReady>
     }
 
     /// Add a personal token to use for authentication.
-    pub fn personal_token(mut self, token: String) -> Self {
-        self.config.auth = Auth::PersonalToken(SecretString::new(token));
+    pub fn personal_token<S: Into<SecretString>>(mut self, token: S) -> Self {
+        self.config.auth = Auth::PersonalToken(token.into());
         self
     }
 
@@ -524,8 +612,8 @@ impl OctocrabBuilder<NoSvc, DefaultOctocrabBuilderConfig, NoAuth, NotLayerReady>
     }
 
     /// Authenticate with a user access token.
-    pub fn user_access_token(mut self, token: String) -> Self {
-        self.config.auth = Auth::UserAccessToken(SecretString::new(token));
+    pub fn user_access_token<S: Into<SecretString>>(mut self, token: S) -> Self {
+        self.config.auth = Auth::UserAccessToken(token.into());
         self
     }
 
@@ -540,21 +628,34 @@ impl OctocrabBuilder<NoSvc, DefaultOctocrabBuilderConfig, NoAuth, NotLayerReady>
         Ok(self)
     }
 
+    /// Set the base upload url for `Octocrab`.
+    pub fn upload_uri(mut self, upload_uri: impl TryInto<Uri>) -> Result<Self> {
+        self.config.upload_uri = Some(
+            upload_uri
+                .try_into()
+                .map_err(|_| UriParseError {})
+                .context(UriParseSnafu)?,
+        );
+        Ok(self)
+    }
+
     #[cfg(feature = "retry")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "retry")))]
     pub fn set_connector_retry_service<S>(
         &self,
-        connector: hyper::Client<S, String>,
-    ) -> Retry<RetryConfig, hyper::Client<S, String>> {
+        connector: hyper_util::client::legacy::Client<S, OctoBody>,
+    ) -> Retry<RetryConfig, hyper_util::client::legacy::Client<S, OctoBody>> {
         let retry_layer = RetryLayer::new(self.config.retry_config.clone());
 
         retry_layer.layer(connector)
     }
 
     #[cfg(feature = "timeout")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "timeout")))]
     pub fn set_connect_timeout_service<T>(&self, connector: T) -> TimeoutConnector<T>
     where
         T: Service<Uri> + Send,
-        T::Response: AsyncRead + AsyncWrite + Send + Unpin,
+        T::Response: hyper::rt::Read + hyper::rt::Write + Send + Unpin,
         T::Future: Send + 'static,
         T::Error: Into<BoxError>,
     {
@@ -567,10 +668,12 @@ impl OctocrabBuilder<NoSvc, DefaultOctocrabBuilderConfig, NoAuth, NotLayerReady>
     }
 
     /// Build a [`Client`] instance with the current [`Service`] stack.
+    #[cfg(feature = "default-client")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "default-client")))]
     pub fn build(self) -> Result<Octocrab> {
-        let client: hyper::Client<_, String> = {
+        let client: hyper_util::client::legacy::Client<_, OctoBody> = {
             #[cfg(all(not(feature = "opentls"), not(feature = "rustls")))]
-            let mut connector = HttpConnector::new();
+            let mut connector = hyper::client::conn::http1::HttpConnector::new();
 
             #[cfg(all(feature = "rustls", not(feature = "opentls")))]
             let connector = {
@@ -578,7 +681,10 @@ impl OctocrabBuilder<NoSvc, DefaultOctocrabBuilderConfig, NoAuth, NotLayerReady>
                 #[cfg(feature = "rustls-webpki-tokio")]
                 let builder = builder.with_webpki_roots();
                 #[cfg(not(feature = "rustls-webpki-tokio"))]
-                let builder = builder.with_native_roots(); // enabled the `rustls-native-certs` feature in hyper-rustls
+                let builder = builder
+                    .with_native_roots()
+                    .map_err(Into::into)
+                    .context(error::OtherSnafu)?; // enabled the `rustls-native-certs` feature in hyper-rustls
 
                 builder
                     .https_or_http() //  Disable .https_only() during tests until: https://github.com/LukeMathWalker/wiremock-rs/issues/58 is resolved. Alternatively we can use conditional compilation to only enable this feature in tests, but it becomes rather ugly with integration tests.
@@ -592,7 +698,8 @@ impl OctocrabBuilder<NoSvc, DefaultOctocrabBuilderConfig, NoAuth, NotLayerReady>
             #[cfg(feature = "timeout")]
             let connector = self.set_connect_timeout_service(connector);
 
-            hyper::Client::builder().build(connector)
+            hyper_util::client::legacy::Client::builder(hyper_util::rt::TokioExecutor::new())
+                .build(connector)
         };
 
         #[cfg(feature = "retry")]
@@ -600,7 +707,7 @@ impl OctocrabBuilder<NoSvc, DefaultOctocrabBuilderConfig, NoAuth, NotLayerReady>
 
         #[cfg(feature = "tracing")]
         let client = TraceLayer::new_for_http()
-            .make_span_with(|req: &Request<String>| {
+            .make_span_with(|req: &Request<OctoBody>| {
                 tracing::debug_span!(
                     "HTTP",
                      http.method = %req.method(),
@@ -611,11 +718,11 @@ impl OctocrabBuilder<NoSvc, DefaultOctocrabBuilderConfig, NoAuth, NotLayerReady>
                      otel.status_code = tracing::field::Empty,
                 )
             })
-            .on_request(|_req: &Request<String>, _span: &Span| {
+            .on_request(|_req: &Request<OctoBody>, _span: &Span| {
                 tracing::debug!("requesting");
             })
             .on_response(
-                |res: &Response<hyper::Body>, _latency: Duration, span: &Span| {
+                |res: &Response<hyper::body::Incoming>, _latency: Duration, span: &Span| {
                     let status = res.status();
                     span.record("http.status_code", status.as_u16());
                     if status.is_client_error() || status.is_server_error() {
@@ -664,27 +771,22 @@ impl OctocrabBuilder<NoSvc, DefaultOctocrabBuilderConfig, NoAuth, NotLayerReady>
             ));
         }
 
-        let auth_state = match self.config.auth {
-            Auth::None => AuthState::None,
-            Auth::Basic { username, password } => AuthState::BasicAuth { username, password },
-            Auth::PersonalToken(token) => {
-                hmap.push((
-                    http::header::AUTHORIZATION,
-                    format!("Bearer {}", token.expose_secret()).parse().unwrap(),
-                ));
-                AuthState::None
+        let (auth_header, auth_state): (Option<HeaderValue>, _) = match self.config.auth {
+            Auth::None => (None, AuthState::None),
+            Auth::Basic { username, password } => {
+                (None, AuthState::BasicAuth { username, password })
             }
-            Auth::UserAccessToken(token) => {
-                hmap.push((
-                    http::header::AUTHORIZATION,
-                    format!("Bearer {}", token.expose_secret()).parse().unwrap(),
-                ));
-                AuthState::None
-            }
-            Auth::App(app_auth) => AuthState::App(app_auth),
-            Auth::OAuth(device) => {
-                hmap.push((
-                    http::header::AUTHORIZATION,
+            Auth::PersonalToken(token) => (
+                Some(format!("Bearer {}", token.expose_secret()).parse().unwrap()),
+                AuthState::None,
+            ),
+            Auth::UserAccessToken(token) => (
+                Some(format!("Bearer {}", token.expose_secret()).parse().unwrap()),
+                AuthState::None,
+            ),
+            Auth::App(app_auth) => (None, AuthState::App(app_auth)),
+            Auth::OAuth(device) => (
+                Some(
                     format!(
                         "{} {}",
                         device.token_type,
@@ -692,9 +794,9 @@ impl OctocrabBuilder<NoSvc, DefaultOctocrabBuilderConfig, NoAuth, NotLayerReady>
                     )
                     .parse()
                     .unwrap(),
-                ));
-                AuthState::None
-            }
+                ),
+                AuthState::None,
+            ),
         };
 
         for (key, value) in self.config.extra_headers.iter() {
@@ -709,17 +811,29 @@ impl OctocrabBuilder<NoSvc, DefaultOctocrabBuilderConfig, NoAuth, NotLayerReady>
         let client = ExtraHeadersLayer::new(Arc::new(hmap)).layer(client);
 
         let client = MapResponseBodyLayer::new(|body| {
-            Box::new(http_body::Body::map_err(body, BoxError::from)) as Box<DynBody>
+            BodyExt::map_err(body, |e| HyperSnafu.into_error(e)).boxed()
         })
         .layer(client);
 
-        let uri = self
+        let base_uri = self
             .config
             .base_uri
             .clone()
             .unwrap_or_else(|| Uri::from_str(GITHUB_BASE_URI).unwrap());
 
-        let client = BaseUriLayer::new(uri).layer(client);
+        let upload_uri = self
+            .config
+            .upload_uri
+            .clone()
+            .unwrap_or_else(|| Uri::from_str(GITHUB_BASE_UPLOAD_URI).unwrap());
+
+        let client = BaseUriLayer::new(base_uri.clone()).layer(client);
+
+        let client = AuthHeaderLayer::new(auth_header, base_uri, upload_uri).layer(client);
+
+        if let Some(executor) = self.executor {
+            return Ok(Octocrab::new_with_executor(client, auth_state, executor));
+        }
 
         Ok(Octocrab::new(client, auth_state))
     }
@@ -736,6 +850,7 @@ pub struct DefaultOctocrabBuilderConfig {
     #[cfg(feature = "timeout")]
     write_timeout: Option<Duration>,
     base_uri: Option<Uri>,
+    upload_uri: Option<Uri>,
     #[cfg(feature = "retry")]
     retry_config: RetryConfig,
 }
@@ -753,6 +868,7 @@ impl Default for DefaultOctocrabBuilderConfig {
             #[cfg(feature = "timeout")]
             write_timeout: None,
             base_uri: None,
+            upload_uri: None,
             #[cfg(feature = "retry")]
             retry_config: RetryConfig::Simple(3),
         }
@@ -765,8 +881,6 @@ impl DefaultOctocrabBuilderConfig {
     }
 }
 
-pub type DynBody = dyn http_body::Body<Data = Bytes, Error = BoxError> + Send + Unpin;
-
 #[derive(Debug, Clone)]
 struct CachedTokenInner {
     expiration: Option<DateTime<Utc>>,
@@ -812,9 +926,8 @@ impl CachedToken {
         self.valid_token_with_buffer(chrono::Duration::seconds(30))
     }
 
-    fn set(&self, token: String, expiration: Option<DateTime<Utc>>) {
-        *self.0.write().unwrap() =
-            Some(CachedTokenInner::new(SecretString::new(token), expiration));
+    fn set<S: Into<SecretString>>(&self, token: S, expiration: Option<DateTime<Utc>>) {
+        *self.0.write().unwrap() = Some(CachedTokenInner::new(token.into(), expiration));
     }
 }
 
@@ -870,11 +983,16 @@ pub enum AuthState {
         /// The cached access token, if any
         token: CachedToken,
     },
+    /// Access token based authentication.
+    AccessToken {
+        /// The access token
+        token: SecretString,
+    },
 }
 
 pub type OctocrabService = Buffer<
-    BoxService<http::Request<String>, http::Response<hyper::Body>, BoxError>,
-    http::Request<String>,
+    http::Request<OctoBody>,
+    <BoxService<http::Request<OctoBody>, http::Response<BoxBody<Bytes, Error>>, BoxError> as tower::Service<http::Request<OctoBody>>>::Future
 >;
 
 /// The GitHub API client.
@@ -896,6 +1014,8 @@ impl fmt::Debug for Octocrab {
 /// - `base_uri`: `https://api.github.com`
 /// - `auth`: `None`
 /// - `client`: http client with the `octocrab` user agent.
+#[cfg(feature = "default-client")]
+#[cfg_attr(docsrs, doc(cfg(feature = "default-client")))]
 impl Default for Octocrab {
     fn default() -> Self {
         OctocrabBuilder::default().build().unwrap()
@@ -911,20 +1031,36 @@ impl Octocrab {
     }
 
     /// Creates a new `Octocrab`.
-    fn new<S, B>(service: S, auth_state: AuthState) -> Self
+    fn new<S>(service: S, auth_state: AuthState) -> Self
     where
-        S: Service<Request<String>, Response = Response<B>> + Send + 'static,
+        S: Service<Request<OctoBody>, Response = Response<BoxBody<Bytes, crate::Error>>>
+            + Send
+            + 'static,
         S::Future: Send + 'static,
         S::Error: Into<BoxError>,
-        B: http_body::Body<Data = bytes::Bytes> + Send + 'static,
-        B::Error: Into<BoxError>,
     {
-        // Transform response body to `hyper::Body` and use type erased error to avoid type parameters.
-        let service = MapResponseBodyLayer::new(|b: B| Body::wrap_stream(b.into_stream()))
-            .layer(service)
-            .map_err(|e| e.into());
+        let service = Buffer::new(BoxService::new(service.map_err(Into::into)), 1024);
 
-        let service = Buffer::new(BoxService::new(service), 1024);
+        Self {
+            client: service,
+            auth_state,
+        }
+    }
+
+    /// Creates a new `Octocrab` with a custom executor
+    fn new_with_executor<S>(service: S, auth_state: AuthState, executor: Executor) -> Self
+    where
+        S: Service<Request<OctoBody>, Response = Response<BoxBody<Bytes, crate::Error>>>
+            + Send
+            + 'static,
+        S::Future: Send + 'static,
+        S::Error: Into<BoxError>,
+    {
+        // Use Buffer pair to return the background worker
+        let (service, worker) = Buffer::pair(BoxService::new(service.map_err(Into::into)), 1024);
+
+        // Execute the background worker with the custom executor
+        executor(Box::pin(worker));
 
         Self {
             client: service,
@@ -939,20 +1075,22 @@ impl Octocrab {
     /// then obtain an installation ID, and then pass that here to
     /// obtain a new `Octocrab` with which you can make API calls
     /// with the permissions of that installation.
-    pub fn installation(&self, id: InstallationId) -> Octocrab {
+    pub fn installation(&self, id: InstallationId) -> Result<Octocrab> {
         let app_auth = if let AuthState::App(ref app_auth) = self.auth_state {
             app_auth.clone()
         } else {
-            panic!("Github App authorization is required to target an installation");
+            return Err(Error::Installation {
+                backtrace: Backtrace::capture(),
+            });
         };
-        Octocrab {
+        Ok(Octocrab {
             client: self.client.clone(),
             auth_state: AuthState::Installation {
                 app: app_auth,
                 installation: id,
                 token: CachedToken::default(),
             },
-        }
+        })
     }
 
     /// Similar to `installation`, but also eagerly caches the installation
@@ -965,10 +1103,23 @@ impl Octocrab {
         &self,
         id: InstallationId,
     ) -> Result<(Octocrab, SecretString)> {
-        let crab = self.installation(id);
+        let crab = self.installation(id)?;
         let token = crab.request_installation_auth_token().await?;
         Ok((crab, token))
     }
+
+    /// Returns a new `Octocrab` based on the current builder but
+    /// authorizing via an access token.
+    ///
+    /// See also https://docs.github.com/en/apps/creating-github-apps/authenticating-with-a-github-app/generating-a-user-access-token-for-a-github-app
+    pub fn user_access_token<S: Into<SecretString>>(&self, token: S) -> Result<Self> {
+        Ok(Octocrab {
+            client: self.client.clone(),
+            auth_state: AuthState::AccessToken {
+                token: token.into(),
+            },
+        })
+    }
 }
 
 /// # GitHub API Methods
@@ -1008,7 +1159,32 @@ impl Octocrab {
         owner: impl Into<String>,
         repo: impl Into<String>,
     ) -> issues::IssueHandler {
-        issues::IssueHandler::new(self, owner.into(), repo.into())
+        issues::IssueHandler::new(self, RepoRef::ByOwnerAndName(owner.into(), repo.into()))
+    }
+
+    /// Creates a [`issues::IssueHandler`] for the repo specified at repository ID,
+    /// that allows you to access GitHub's issues API.
+    pub fn issues_by_id(&self, id: impl Into<RepositoryId>) -> issues::IssueHandler {
+        issues::IssueHandler::new(self, RepoRef::ById(id.into()))
+    }
+
+    /// Creates a [`code_scanning::CodeSCanningHandler`] for the repo specified at `owner/repo`,
+    /// that allows you to access GitHub's Code scanning API.
+    pub fn code_scannings(
+        &self,
+        owner: impl Into<String>,
+        repo: impl Into<String>,
+    ) -> code_scannings::CodeScanningHandler {
+        code_scannings::CodeScanningHandler::new(self, owner.into(), Option::from(repo.into()))
+    }
+
+    /// Creates a [`code_scanning::CodeSCanningHandler`] for the org specified at `owner`,
+    /// that allows you to access GitHub's Code scanning API.
+    pub fn code_scannings_organisation(
+        &self,
+        owner: impl Into<String>,
+    ) -> code_scannings::CodeScanningHandler {
+        code_scannings::CodeScanningHandler::new(self, owner.into(), None)
     }
 
     /// Creates a [`commits::CommitHandler`] for the repo specified at `owner/repo`,
@@ -1049,7 +1225,13 @@ impl Octocrab {
     /// Creates a [`repos::RepoHandler`] for the repo specified at `owner/repo`,
     /// that allows you to access GitHub's repository API.
     pub fn repos(&self, owner: impl Into<String>, repo: impl Into<String>) -> repos::RepoHandler {
-        repos::RepoHandler::new(self, owner.into(), repo.into())
+        repos::RepoHandler::new(self, RepoRef::ByOwnerAndName(owner.into(), repo.into()))
+    }
+
+    /// Creates a [`repos::RepoHandler`] for the repo specified at repository ID,
+    /// that allows you to access GitHub's repository API.
+    pub fn repos_by_id(&self, id: impl Into<RepositoryId>) -> repos::RepoHandler {
+        repos::RepoHandler::new(self, RepoRef::ById(id.into()))
     }
 
     /// Creates a [`projects::ProjectHandler`] that allows you to access GitHub's
@@ -1070,9 +1252,14 @@ impl Octocrab {
         teams::TeamHandler::new(self, owner.into())
     }
 
-    /// Creates a [`users::UserHandler`] for the specified user
+    /// Creates a [`users::UserHandler`] for the specified user using the user name
     pub fn users(&self, user: impl Into<String>) -> users::UserHandler {
-        users::UserHandler::new(self, user.into())
+        users::UserHandler::new(self, UserRef::ByString(user.into()))
+    }
+
+    /// Creates a [`users::UserHandler`] for the specified user using the user ID
+    pub fn users_by_id(&self, user: impl Into<UserId>) -> users::UserHandler {
+        users::UserHandler::new(self, UserRef::ById(user.into()))
     }
 
     /// Creates a [`workflows::WorkflowsHandler`] for the specified repository that allows
@@ -1110,6 +1297,11 @@ impl Octocrab {
     pub fn ratelimit(&self) -> ratelimit::RateLimitHandler {
         ratelimit::RateLimitHandler::new(self)
     }
+
+    /// Creates a [`hooks::HooksHandler`] that returns the API hooks
+    pub fn hooks(&self, owner: impl Into<String>) -> hooks::HooksHandler {
+        hooks::HooksHandler::new(self, owner.into())
+    }
 }
 
 /// # GraphQL API.
@@ -1163,7 +1355,7 @@ impl Octocrab {
         &self,
         uri: impl TryInto<http::Uri>,
         body: Option<&P>,
-    ) -> Result<http::Response<hyper::Body>> {
+    ) -> Result<http::Response<BoxBody<Bytes, crate::Error>>> {
         let uri = uri
             .try_into()
             .map_err(|_| UriParseError {})
@@ -1185,7 +1377,10 @@ impl Octocrab {
     }
 
     /// Send a `GET` request with no additional post-processing.
-    pub async fn _get(&self, uri: impl TryInto<Uri>) -> Result<http::Response<hyper::Body>> {
+    pub async fn _get(
+        &self,
+        uri: impl TryInto<Uri>,
+    ) -> Result<http::Response<BoxBody<Bytes, crate::Error>>> {
         self._get_with_headers(uri, None).await
     }
 
@@ -1215,8 +1410,11 @@ impl Octocrab {
         uri
     }
 
-    pub async fn body_to_string(&self, res: http::Response<Body>) -> Result<String> {
-        let body_bytes = body::to_bytes(res.into_body()).await.context(HyperSnafu)?;
+    pub async fn body_to_string(
+        &self,
+        res: http::Response<BoxBody<Bytes, crate::Error>>,
+    ) -> Result<String> {
+        let body_bytes = res.into_body().collect().await?.to_bytes();
         String::from_utf8(body_bytes.to_vec()).context(InvalidUtf8Snafu)
     }
 
@@ -1244,7 +1442,7 @@ impl Octocrab {
         &self,
         uri: impl TryInto<Uri>,
         headers: Option<http::header::HeaderMap>,
-    ) -> Result<http::Response<hyper::Body>> {
+    ) -> Result<http::Response<BoxBody<Bytes, crate::Error>>> {
         let uri = uri
             .try_into()
             .map_err(|_| UriParseError {})
@@ -1278,7 +1476,7 @@ impl Octocrab {
         &self,
         uri: impl TryInto<Uri>,
         body: Option<&B>,
-    ) -> Result<http::Response<hyper::Body>> {
+    ) -> Result<http::Response<BoxBody<Bytes, crate::Error>>> {
         let uri = uri
             .try_into()
             .map_err(|_| UriParseError {})
@@ -1307,7 +1505,7 @@ impl Octocrab {
         &self,
         uri: impl TryInto<Uri>,
         body: Option<&B>,
-    ) -> Result<http::Response<hyper::Body>> {
+    ) -> Result<http::Response<BoxBody<Bytes, crate::Error>>> {
         let uri = uri
             .try_into()
             .map_err(|_| UriParseError {})
@@ -1321,7 +1519,7 @@ impl Octocrab {
         &self,
         mut builder: Builder,
         body: Option<&B>,
-    ) -> Result<http::Request<String>> {
+    ) -> Result<http::Request<OctoBody>> {
         // Since Octocrab doesn't require streamable bodies(aka, file upload) because it is serde::Serialize),
         // we can just use String body, since it is both http_body::Body(required by Hyper::Client), and Clone(required by BoxService).
 
@@ -1330,12 +1528,15 @@ impl Octocrab {
 
         if let Some(body) = body {
             builder = builder.header(http::header::CONTENT_TYPE, "application/json");
-            let request = builder
-                .body(serde_json::to_string(body).context(SerdeSnafu)?)
-                .context(HttpSnafu)?;
+            let serialized = serde_json::to_string(body).context(SerdeSnafu)?;
+            let body: OctoBody = serialized.into();
+            let request = builder.body(body).context(HttpSnafu)?;
             Ok(request)
         } else {
-            Ok(builder.body(String::new()).context(HttpSnafu)?)
+            Ok(builder
+                .header(http::header::CONTENT_LENGTH, "0")
+                .body(OctoBody::empty())
+                .context(HttpSnafu)?)
         }
     }
 
@@ -1358,7 +1559,7 @@ impl Octocrab {
         &self,
         uri: impl TryInto<Uri>,
         body: Option<&B>,
-    ) -> Result<http::Response<hyper::Body>> {
+    ) -> Result<http::Response<BoxBody<Bytes, crate::Error>>> {
         let uri = uri
             .try_into()
             .map_err(|_| UriParseError {})
@@ -1378,7 +1579,9 @@ impl Octocrab {
         {
             (app, installation, token)
         } else {
-            panic!("Installation not configured");
+            return Err(Error::Installation {
+                backtrace: Backtrace::capture(),
+            });
         };
         let mut request = Builder::new();
         let mut sensitive_value =
@@ -1393,11 +1596,11 @@ impl Octocrab {
 
         sensitive_value.set_sensitive(true);
         request = request
-            .header(hyper::header::AUTHORIZATION, sensitive_value)
+            .header(http::header::AUTHORIZATION, sensitive_value)
             .method(http::Method::POST)
             .uri(uri);
         let response = self
-            .send(request.body(String::new()).context(HttpSnafu)?)
+            .send(request.body("{}".into()).context(HttpSnafu)?)
             .await?;
         let _status = response.status();
 
@@ -1409,7 +1612,7 @@ impl Octocrab {
             .map(|time| {
                 DateTime::<Utc>::from_str(&time).map_err(|e| error::Error::Other {
                     source: Box::new(e),
-                    backtrace: snafu::Backtrace::generate(),
+                    backtrace: snafu::Backtrace::capture(),
                 })
             })
             .transpose()?;
@@ -1419,13 +1622,16 @@ impl Octocrab {
 
         token.set(token_object.token.clone(), expiration);
 
-        Ok(SecretString::new(token_object.token))
+        Ok(SecretString::from(token_object.token))
     }
 
     /// Send the given request to the underlying service
-    pub async fn send(&self, request: Request<String>) -> Result<http::Response<hyper::Body>> {
+    pub async fn send(
+        &self,
+        request: Request<OctoBody>,
+    ) -> Result<http::Response<BoxBody<Bytes, crate::Error>>> {
         let mut svc = self.client.clone();
-        let response: Response<Body> = svc
+        let response: Response<BoxBody<Bytes, crate::Error>> = svc
             .ready()
             .await
             .context(ServiceSnafu)?
@@ -1448,9 +1654,10 @@ impl Octocrab {
     /// Execute the given `request` using octocrab's Client.
     pub async fn execute(
         &self,
-        request: http::Request<String>,
-    ) -> Result<http::Response<hyper::Body>> {
+        request: http::Request<impl Into<OctoBody>>,
+    ) -> Result<http::Response<BoxBody<Bytes, crate::Error>>> {
         let (mut parts, body) = request.into_parts();
+        let body: OctoBody = body.into();
         // Saved request that we can retry later if necessary
         let auth_header: Option<HeaderValue> = match self.auth_state {
             AuthState::None => None,
@@ -1488,13 +1695,24 @@ impl Octocrab {
                         .context(HttpSnafu)?,
                 )
             }
+            AuthState::AccessToken { ref token } => Some(
+                HeaderValue::from_str(format!("Bearer {}", token.expose_secret()).as_str())
+                    .map_err(http::Error::from)
+                    .context(HttpSnafu)?,
+            ),
         };
 
         if let Some(mut auth_header) = auth_header {
-            auth_header.set_sensitive(true);
-            parts
-                .headers
-                .insert(hyper::header::AUTHORIZATION, auth_header);
+            // Only set the auth_header if the authority (host) is empty (destined for
+            // GitHub). Otherwise, leave it off as we could have been redirected
+            // away from GitHub (via follow_location_to_data()), and we don't
+            // want to give our credentials to third-party services.
+            if parts.uri.authority().is_none() {
+                auth_header.set_sensitive(true);
+                parts
+                    .headers
+                    .insert(http::header::AUTHORIZATION, auth_header);
+            }
         }
 
         let request = http::Request::from_parts(parts, body);
@@ -1512,8 +1730,8 @@ impl Octocrab {
 
     pub async fn follow_location_to_data(
         &self,
-        response: http::Response<hyper::Body>,
-    ) -> crate::Result<http::Response<hyper::Body>> {
+        response: http::Response<BoxBody<Bytes, Error>>,
+    ) -> crate::Result<http::Response<BoxBody<Bytes, crate::Error>>> {
         if let Some(redirect) = response.headers().get(http::header::LOCATION) {
             let location = redirect.to_str().expect("Location URL not valid str");
 
