@@ -8,9 +8,14 @@ use chrono::{DateTime, Utc};
 use serde::{de, Deserialize, Deserializer, Serialize};
 use url::Url;
 
+use crate::params::users::emails::EmailVisibilityState;
+pub use apps::App;
+
+pub mod actions;
 pub mod activity;
 pub mod apps;
 pub mod checks;
+pub mod code_scannings;
 pub mod commits;
 pub mod events;
 pub mod gists;
@@ -27,8 +32,6 @@ pub mod workflows;
 
 mod date_serde;
 
-pub use apps::App;
-
 type BaseIdType = u64;
 
 macro_rules! id_type {
@@ -104,12 +107,14 @@ id_type!(
     CardId,
     CheckSuiteId,
     CheckRunId,
+    CodeScanningId,
     CommentId,
     InstallationId,
     IssueEventId,
     IssueId,
     JobId,
     HookId,
+    HookDeliveryId,
     LabelId,
     MilestoneId,
     NotificationId,
@@ -123,6 +128,9 @@ id_type!(
     RepositoryId,
     ReviewId,
     RunId,
+    RunnerId,
+    RunnerGroupId,
+    RunnerLabelId,
     StatusId,
     TeamId,
     TimelineEventId,
@@ -177,10 +185,18 @@ pub enum Event {
     AddedToProject,
     /// The issue or pull request was assigned to a user.
     Assigned,
+    /// Auto merge was disabled for a pull request.
+    AutoMergeDisabled,
+    /// Auto merge was enabled for a pull request.
+    AutoMergeEnabled,
     /// GitHub unsuccessfully attempted to automatically change the base branch of the pull request.
     AutomaticBaseChangeFailed,
     /// GitHub successfully attempted to automatically change the base branch of the pull request.
     AutomaticBaseChangeSucceeded,
+    /// Auto Rebase Enable
+    AutoRebaseEnabled,
+    /// Auto Squash Enable
+    AutoSquashEnabled,
     /// The base reference branch of the pull request changed.
     BaseRefChanged,
     /// Not documented in the Github issue events documentation.
@@ -411,6 +427,140 @@ pub struct Author {
     pub email: Option<String>,
 }
 
+/// If a string is empty then deserialize it as none
+fn empty_string_is_none<'de, D>(deserializer: D) -> Result<Option<String>, D::Error>
+where
+    D: Deserializer<'de>,
+{
+    // try to deserialize our input string
+    let cast = String::deserialize(deserializer)?;
+    // if this string is empty then return None
+    if cast.is_empty() {
+        Ok(None)
+    } else {
+        Ok(Some(cast))
+    }
+}
+
+/// The full profile for a user
+#[derive(Debug, Clone, Hash, Eq, PartialEq, Serialize, Deserialize)]
+#[non_exhaustive]
+pub struct UserProfile {
+    pub login: String,
+    pub id: UserId,
+    pub node_id: String,
+    pub avatar_url: Url,
+    pub gravatar_id: String,
+    pub url: Url,
+    pub html_url: Url,
+    pub followers_url: Url,
+    pub following_url: Url,
+    pub gists_url: Url,
+    pub starred_url: Url,
+    pub subscriptions_url: Url,
+    pub organizations_url: Url,
+    pub repos_url: Url,
+    pub events_url: Url,
+    pub received_events_url: Url,
+    pub r#type: String,
+    pub site_admin: bool,
+    pub name: Option<String>,
+    pub company: Option<String>,
+    #[serde(deserialize_with = "empty_string_is_none")]
+    pub blog: Option<String>,
+    pub location: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub email: Option<String>,
+    pub hireable: Option<bool>,
+    pub bio: Option<String>,
+    pub twitter_username: Option<String>,
+    pub public_repos: u64,
+    pub public_gists: u64,
+    pub followers: u64,
+    pub following: u64,
+    pub created_at: DateTime<Utc>,
+    pub updated_at: DateTime<Utc>,
+}
+
+/// The simple profile for a GitHub user
+#[derive(Debug, Clone, Hash, Eq, PartialEq, Serialize, Deserialize)]
+#[non_exhaustive]
+pub struct SimpleUser {
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub name: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub email: Option<String>,
+    pub login: String,
+    pub id: UserId,
+    pub node_id: String,
+    pub avatar_url: Url,
+    pub gravatar_id: String,
+    pub url: Url,
+    pub html_url: Url,
+    pub followers_url: Url,
+    pub following_url: Url,
+    pub gists_url: Url,
+    pub starred_url: Url,
+    pub subscriptions_url: Url,
+    pub organizations_url: Url,
+    pub repos_url: Url,
+    pub events_url: Url,
+    pub received_events_url: Url,
+    pub r#type: String,
+    pub site_admin: bool,
+    pub starred_at: Option<DateTime<Utc>>,
+}
+
+/// A user that is following another user
+#[derive(Debug, Clone, Hash, Eq, PartialEq, Serialize, Deserialize)]
+#[non_exhaustive]
+pub struct Follower {
+    pub login: String,
+    pub id: UserId,
+    pub node_id: String,
+    pub avatar_url: Url,
+    pub gravatar_id: String,
+    pub url: Url,
+    pub html_url: Url,
+    pub followers_url: Url,
+    pub following_url: Url,
+    pub gists_url: Url,
+    pub starred_url: Url,
+    pub subscriptions_url: Url,
+    pub organizations_url: Url,
+    pub repos_url: Url,
+    pub events_url: Url,
+    pub received_events_url: Url,
+    pub r#type: String,
+    pub site_admin: bool,
+    pub patch_url: Option<String>,
+}
+
+/// A user that is being followed by another user
+#[derive(Debug, Clone, Hash, Eq, PartialEq, Serialize, Deserialize)]
+#[non_exhaustive]
+pub struct Followee {
+    pub login: String,
+    pub id: UserId,
+    pub node_id: String,
+    pub avatar_url: Url,
+    pub gravatar_id: String,
+    pub url: Url,
+    pub html_url: Url,
+    pub followers_url: Url,
+    pub following_url: Url,
+    pub gists_url: Url,
+    pub starred_url: Url,
+    pub subscriptions_url: Url,
+    pub organizations_url: Url,
+    pub repos_url: Url,
+    pub events_url: Url,
+    pub received_events_url: Url,
+    pub r#type: String,
+    pub site_admin: bool,
+    pub patch_url: Option<String>,
+}
+
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "SCREAMING_SNAKE_CASE")]
 #[non_exhaustive]
@@ -435,6 +585,14 @@ pub struct Collaborator {
     pub permissions: Permissions,
 }
 
+#[derive(Debug, Clone, Eq, PartialEq, Serialize, Deserialize)]
+#[non_exhaustive]
+pub struct Contributor {
+    #[serde(flatten)]
+    pub author: Author,
+    pub contributions: u32,
+}
+
 #[derive(Debug, Clone, Hash, Eq, PartialEq, Serialize, Deserialize)]
 #[non_exhaustive]
 pub struct StarGazer {
@@ -940,5 +1098,89 @@ pub struct Rate {
     pub limit: usize,
     pub used: usize,
     pub remaining: usize,
-    pub reset: usize,
+    pub reset: u64,
+}
+
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct UserEmailInfo {
+    pub email: String,
+    pub primary: bool,
+    pub verified: bool,
+    pub visibility: EmailVisibilityState,
+}
+
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct VerifiedEmailInfo {
+    pub email: String,
+    pub verified: bool,
+}
+
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct SubKeyInfo {
+    pub id: u64,
+    pub primary_key_id: u64,
+    pub key_id: String,
+    pub public_key: String,
+    pub emails: Vec<VerifiedEmailInfo>,
+    pub subkeys: Option<Vec<SubKeyInfo>>,
+    pub can_sign: bool,
+    pub can_encrypt_comms: bool,
+    pub can_encrypt_storage: bool,
+    pub can_certify: bool,
+    pub created_at: String,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub expires_at: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub raw_key: Option<String>,
+    pub revoked: bool,
+}
+
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct GpgKey {
+    pub id: u64,
+    pub name: String,
+    pub primary_key_id: u64,
+    pub key_id: String,
+    pub public_key: String,
+    pub emails: Vec<VerifiedEmailInfo>,
+    pub subkeys: Vec<SubKeyInfo>,
+    pub can_sign: bool,
+    pub can_encrypt_comms: bool,
+    pub can_encrypt_storage: bool,
+    pub can_certify: bool,
+    pub created_at: DateTime<Utc>,
+    #[serde(
+        skip_serializing_if = "Option::is_none",
+        default,
+        deserialize_with = "date_serde::deserialize_opt"
+    )]
+    pub expires_at: Option<DateTime<Utc>>,
+    pub revoked: bool,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub raw_key: Option<String>,
+}
+
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct GitSshKey {
+    pub key: String,
+    pub id: u64,
+    pub url: String,
+    pub title: String,
+    pub created_at: DateTime<Utc>,
+    pub verified: bool,
+    pub read_only: bool,
+}
+
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct SocialAccount {
+    pub provider: String,
+    pub url: String,
+}
+
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct SshSigningKey {
+    pub key: String,
+    pub id: u64,
+    pub title: String,
+    pub created_at: DateTime<Utc>,
 }
