@@ -1,5 +1,7 @@
 use serde::{Deserialize, Serialize};
 
+use crate::models::Author;
+
 use super::OldValue;
 
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
@@ -32,4 +34,18 @@ pub struct RepositoryWebhookEventChanges {
     pub description: Option<OldValue<Option<String>>>,
     pub homepage: Option<OldValue<Option<String>>>,
     pub topics: Option<OldValue<Option<Vec<String>>>>,
+    pub owner: Option<OldValue<RepositoryWebhookEventChangesOwner>>,
+    pub repository: Option<RepositoryWebhookEventChangesRepository>,
+}
+
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[non_exhaustive]
+pub struct RepositoryWebhookEventChangesOwner {
+    pub user: Author,
+}
+
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[non_exhaustive]
+pub struct RepositoryWebhookEventChangesRepository {
+    pub name: Option<OldValue<String>>,
 }
